#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .PreView import PreView
from .EditView import EditView
from .ToolBarButton import ToolBarButton, ToolBarMenuInfo
from .LineNumberArea import LineNumberArea



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from mistune import Markdown, Renderer, escape


class MkRenderer(Renderer):
    def __init__(self, **kwargs):
        Renderer.__init__(self, **kwargs)

    def block_code(self, code, lang=None):
        """Rendering block level code. ``pre > code``.

        :param code: text content of the code block.
        :param lang: language of the given code.
        """

        code = code.rstrip('\n')
        print(code)
        if lang and lang == "mermaid":
            code = escape(code, smart_amp=False)
            return '<div class="mermaid">%s\n</div>\n' % code

        return Renderer.block_code(self, code, lang)


class MkUtuls:
    def __init__(self):
        renderer = MkRenderer(escape=True, hard_wrap=True)
        self.markdown = Markdown(renderer=renderer)

    def parse(self, txt):
        return self.markdown.parse(txt)



from PySide2.QtWidgets import QDialog, QLabel
from PySide2 import QtWidgets
from PySide2.QtCore import QSize,Qt

class LoadingHitDialog(QDialog):
    _instance = None
    _object = None

    @classmethod
    def getInstance(cls):
        if not cls._object:
            cls._object = LoadingHitDialog()
        return cls._object

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(LoadingHitDialog, cls).__new__(cls, *args, **kwargs)

        return cls._instance

    def __init__(self):
        QDialog.__init__(self)
        self.setWindowTitle("温馨提示")
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.initView()
        size = QSize()
        size.setHeight(200)
        size.setWidth(300)
        self.setFixedSize(size)
        self.openCount = 0

    def initView(self):
        self.label = QLabel()
        self.label.setText("处理中,请稍后.....")
        mainLayout = QtWidgets.QGridLayout()
        mainLayout.addWidget(self.label, 0, 0, 1, 1)
        self.setLayout(mainLayout)

    def show(self, text: str = None):
        if text:
            self.label.setText(text)
        self.openCount = self.openCount + 1
        if self.openCount > 1:
            return
        QDialog.show(self)

    def hide(self):
        self.openCount = self.openCount - 1
        if self.openCount < 0:
            self.openCount = 0
        if self.openCount == 0:
            self.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from PySide2.QtWidgets import QDialog, QVBoxLayout, QShortcut, QApplication
from mkedit.core.widgets import PreView
from PySide2.QtGui import QKeySequence
from PySide2.QtCore import QSize


class PrewViewDialog(QDialog):
    def __init__(self, parent=None):
        QDialog.__init__(self, parent)
        self.mainLayout = QVBoxLayout(self)
        self.preView = PreView(self)
        self.mainLayout.addWidget(self.preView)
        self.setLayout(self.mainLayout)
        desktop = QApplication.desktop()
        desktop.width() / 2,
        size = QSize()
        size.setHeight(int(desktop.height() * 0.9))
        size.setWidth(int(desktop.width() * 0.33))
        self.setFixedSize(size)
        shortcut = QShortcut(QKeySequence("Ctrl+W"), self)
        shortcut.activated.connect(self.closePreView)

    def closePreView(self):
        self.close()

    def setPreHtml(self, html):
        self.preView.initHtml()
        self.preView.setPreContent(html)
        self.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PySide2.QtWidgets import QPlainTextEdit, QTextEdit, QShortcut, QMessageBox
from PySide2.QtGui import QKeyEvent, QColor, QTextFormat, QPainter, QPalette, Qt, QTextCursor, QKeySequence, \
    QTextDocument
from .LineNumberArea import LineNumberArea
from PySide2.QtCore import QRect, Signal
import PySide2
from mkedit.core.ImageUpload import imageUploadManager, ImageSMMSSaveStrategy
from mkedit.core.dialog import LoadingHitDialog
import os
from rx import operators as ops, scheduler
from scheduler.PySild2QtScheduler import qtScheduler
from mkedit.core.dialog import SearchHitDialog


class EditView(QPlainTextEdit):
    msgNotifySignal = Signal(str)

    def __init__(self, parent=None):
        QPlainTextEdit.__init__(self, parent)
        self.init()
        self.initEvent()

    def init(self):
        self.haseSave = True
        self.showLineCount = None
        self.lineNumberArea = LineNumberArea(self)
        self.noSelection = False
        self.supportImageSuffix = [".png", '.jpeg', '.jpg', '.svg', '.gif', '.bmp', '.webp', '.icon']
        self.searchDialog = None

    def initEvent(self):
        self.blockCountChanged.connect(self.updateLineNumberAreaWidth)
        self.updateRequest.connect(self.updateLineNumberArea)
        self.updateLineNumberAreaWidth()

        # highlight line
        self.textChanged.connect(self.highlightCurrentLine)
        self.cursorPositionChanged.connect(self.highlightCurrentLine)

        shortcut = QShortcut(QKeySequence("Ctrl+F"), self)
        shortcut.activated.connect(self.search)

    def search(self):
        # 弹出搜索输入弹窗
        if not self.searchDialog:
            self.searchDialog = SearchHitDialog.getInstance()
        else:
            self.searchDialog.close()

        self.searchDialog.init(self)
        self.searchDialog.show()

    def updateLineNumberAreaWidth(self):
        self.setViewportMargins(self.lineNumberAreaWidth(), 0, 0, 0)

    def updateLineNumberArea(self, rect, dy):

        if dy:
            self.lineNumberArea.scroll(0, dy)
        else:
            self.lineNumberArea.update(0, rect.y(), self.lineNumberArea.width(), rect.height())

        if rect.contains(self.viewport().rect()):
            self.updateLineNumberAreaWidth()

    def lineNumberAreaPaintEvent(self, event):

        if self.isReadOnly():
            return

        painter = QPainter(self.lineNumberArea)
        painter.fillRect(event.rect(), QColor(38, 38, 38))

        block = self.firstVisibleBlock()
        blockNumber = block.blockNumber()
        top = int(self.blockBoundingGeometry(block).translated(self.contentOffset()).top())
        bottom = top + int(self.blockBoundingRect(block).height())
        currentLine = self.document().findBlock(self.textCursor().position()).blockNumber()

        painter.setPen(self.palette().color(QPalette.Text))

        while block.isValid() and top <= event.rect().bottom():

            # default grey
            textColor = QColor(155, 155, 155)

            if blockNumber == currentLine and self.hasFocus():
                # current line
                textColor = QColor(255, 170, 0, 255)

            painter.setPen(textColor)

            number = "%s " % str(blockNumber + 1)
            painter.drawText(4, top, self.lineNumberArea.width(), self.fontMetrics().height(), Qt.AlignHCenter,
                             number)

            # Move to the next block
            block = block.next()
            top = bottom
            bottom = top + int(self.blockBoundingRect(block).height())
            blockNumber += 1

    def lineNumberAreaWidth(self):
        digits = 1
        maxNum = max(1, self.blockCount())
        while maxNum >= 10:
            maxNum /= 10
            digits += 1

        space = 7 + self.fontMetrics().width('9') * digits
        return space

    def resizeEvent(self, event):
        QPlainTextEdit.resizeEvent(self, event)

        cr = self.contentsRect()
        self.lineNumberArea.setGeometry(QRect(cr.left(), cr.top(), self.lineNumberAreaWidth(), cr.height()))

    def highlightCurrentLine(self):
        '''
        Highlight currently selected line
        '''

        extraSelections = []

        selection = QTextEdit.ExtraSelection()

        lineColor = QColor(230, 230, 230, 255)

        if not self.hasFocus() or self.isReadOnly():
            lineColor.setAlpha(0)

        selection.format.setBackground(lineColor)
        selection.format.setProperty(QTextFormat.FullWidthSelection, True)
        selection.cursor = self.textCursor()
        selection.cursor.clearSelection()

        extraSelections.append(selection)

        self.setExtraSelections(extraSelections)

    def scrollContentsBy(self, dx: int, dy: int):

        if dy > 0:
            # print("下滑 第一个可见line:%s| 最后一个可见行: %s" % (
            #     self.firstVisibleBlock().blockNumber(), self.getLastVisibleBlock()))
            self.scrollContentEvent(self.firstVisibleBlock().blockNumber(), -1)

        elif dy < 0:
            # print("上滑 第一个可见line:%s| 最后一个可见行: %s" % (
            #     self.firstVisibleBlock().blockNumber(), self.getLastVisibleBlock()))
            self.scrollContentEvent(self.firstVisibleBlock().blockNumber(), 1)

        QPlainTextEdit.scrollContentsBy(self, dx, dy)

    def scrollContentEvent(self, firstVisitLine, up):
        print("scrollCallBack# firstVisitLine= %s and percentage=%s" % (
            firstVisitLine, firstVisitLine / self.blockCount()))
        pass

    def findBlocks(self, first=0, last=None, exclude=[]):
        '''
        Divide text in blocks
        '''

        blocks = []
        if last == None:
            last = self.document().characterCount()
        for pos in range(first, last + 1):
            block = self.document().findBlock(pos)
            if block not in blocks and block not in exclude:
                blocks.append(block)
        return blocks

    def blocks2list(self, blocks, mode=None):
        '''
        Convert a block to a string.
        If a mode is specified, preform custom modification to the text.
        '''

        text = []

        toggle = None

        for block in blocks:

            blockText = block.text()

            # ------------------------------------------------------------------------------------------

            if mode == 'unindent':

                if blockText.startswith(' ' * 4):
                    blockText = blockText[4:]
                    self.lastChar -= 4

                elif blockText.startswith('\t'):
                    blockText = blockText[1:]
                    self.lastChar -= 1

            # ------------------------------------------------------------------------------------------

            elif mode == 'indent':
                blockText = ' ' * 4 + blockText
                self.lastChar += 4

            # ------------------------------------------------------------------------------------------

            elif mode == 'comment':

                unindentedBlockText = blockText.lstrip()
                indents = len(blockText) - len(unindentedBlockText)

                if toggle is None:
                    toggle = not unindentedBlockText.startswith('#')

                # kill comment
                if unindentedBlockText.startswith('# '):
                    unindentedBlockText = unindentedBlockText[2:]

                elif unindentedBlockText.startswith('#'):
                    unindentedBlockText = unindentedBlockText[1:]

                # combine
                blockText = (' ' * indents) + ('# ' * int(toggle)) + unindentedBlockText

            # ------------------------------------------------------------------------------------------

            text.append(blockText)
        return text

    def getCursorInfo(self):

        self.cursor = self.textCursor()

        self.firstChar = self.cursor.selectionStart()
        self.lastChar = self.cursor.selectionEnd()

        self.noSelection = False
        if self.firstChar == self.lastChar:
            self.noSelection = True

        self.originalPosition = self.cursor.position()
        self.cursorBlockPos = self.cursor.positionInBlock()

    def indentation(self, mode):
        '''
        Indent selected
        '''

        self.getCursorInfo()

        # if nothing is selected and mode is set to indent, simply insert as many
        # space as needed to reach the next indentation level.

        if self.noSelection and mode == 'indent':
            remainingSpaces = 4 - (self.cursorBlockPos % 4)
            self.insertPlainText(' ' * remainingSpaces)
            return

        selectedBlocks = self.findBlocks(self.firstChar, self.lastChar)
        beforeBlocks = self.findBlocks(last=self.firstChar - 1, exclude=selectedBlocks)
        afterBlocks = self.findBlocks(first=self.lastChar + 1, exclude=selectedBlocks)

        beforeBlocksText = self.blocks2list(beforeBlocks)
        selectedBlocksText = self.blocks2list(selectedBlocks, mode)
        afterBlocksText = self.blocks2list(afterBlocks)

        combinedText = '\n'.join(beforeBlocksText + selectedBlocksText + afterBlocksText)

        # make sure the line count stays the same
        originalBlockCount = len(self.toPlainText().split('\n'))
        combinedText = '\n'.join(combinedText.split('\n')[:originalBlockCount])

        self.clear()
        self.setPlainText(combinedText)
        self.restoreSelection()

    def restoreSelection(self):
        '''
        Restore the original selection and cursor posiftion modifing the text.
        '''

        if self.noSelection:
            self.cursor.setPosition(self.lastChar)

        # check whether the the original selection was from top to bottom or vice versa
        else:
            if self.originalPosition == self.firstChar:
                first = self.lastChar
                last = self.firstChar
                firstBlockSnap = QTextCursor.EndOfBlock
                lastBlockSnap = QTextCursor.StartOfBlock
            else:
                first = self.firstChar
                last = self.lastChar
                firstBlockSnap = QTextCursor.StartOfBlock
                lastBlockSnap = QTextCursor.EndOfBlock

            self.cursor.setPosition(first)
            self.cursor.movePosition(firstBlockSnap, QTextCursor.MoveAnchor)
            self.cursor.setPosition(last, QTextCursor.KeepAnchor)
            self.cursor.movePosition(lastBlockSnap, QTextCursor.KeepAnchor)

        self.setTextCursor(self.cursor)

    def keyPressEvent(self, event: QKeyEvent):
        # print(event.key())
        # if Tab convert to Space
        if event.key() == 16777217:
            self.indentation('indent')

        # if Shift+Tab remove indent
        elif event.key() == 16777218:
            self.indentation('unindent')
        else:
            QPlainTextEdit.keyPressEvent(self, event)

    def getLastVisibleBlock(self) -> int:
        if not self.showLineCount:
            self.showLineCount = self.getShowLines()
        lastVisibleBlockNumber = int(self.showLineCount) + self.firstVisibleBlock().blockNumber()
        return lastVisibleBlockNumber

    def getShowLines(self) -> int:
        height = self.height()
        lineHeight = self.blockBoundingRect(self.firstVisibleBlock()).height()
        showLineCount = height / lineHeight
        return int(showLineCount)

    def dragEnterEvent(self, event: PySide2.QtGui.QDragEnterEvent):
        print("dragEnterEvent")
        accept = False
        if event.mimeData().hasUrls():
            for url in event.mimeData().urls():
                print(url)
                if url.isLocalFile():
                    suffix = os.path.splitext(url.toLocalFile())[-1]
                    print(suffix)
                    if suffix in self.supportImageSuffix:
                        accept = True
                        break

        if accept:
            event.acceptProposedAction()
        else:
            event.ignore()

    def dropEvent(self, event: PySide2.QtGui.QDropEvent):
        data = event.mimeData()
        if data.hasUrls():
            urls = data.urls()
            if len(urls) <= 0:
                return
            files = list()
            for fileUrl in urls:
                print("fileUrl:%s" % fileUrl)
                if fileUrl.isLocalFile():
                    files.append(fileUrl.toLocalFile())
        self.msgNotifySignal.emit("start:开始上传图片")
        imageUploadManager.saveImage(files).pipe(
            ops.subscribe_on(scheduler.ThreadPoolScheduler()),
            qtScheduler.QtScheduler()
        ).subscribe(on_next=self.handlerUploadResult,
                    on_error=self.handlerUploadError)

    def insertFromMimeData(self, source: PySide2.QtCore.QMimeData):
        if source.hasImage():
            # 获取剪贴板的图片
            self.msgNotifySignal.emit("start:开始上传图片")

            imageUploadManager.saveImageForQMineImage(source).pipe(
                ops.subscribe_on(scheduler.ThreadPoolScheduler()),
                qtScheduler.QtScheduler()
            ).subscribe(on_next=self.handlerUploadResult,
                        on_error=self.handlerUploadError)
            return False
        else:
            return QPlainTextEdit.insertFromMimeData(self, source)

    def handlerUploadError(self, error):
        self.msgNotifySignal.emit("end:%s" % error)
        msgBox = QMessageBox()
        msgBox.setFixedWidth(300)
        msgBox.setFixedHeight(200)
        msgBox.setText("错误提示")
        msgBox.setInformativeText(error)
        msgBox.setStandardButtons(QMessageBox.Yes)
        msgBox.setDefaultButton(QMessageBox.Yes)
        ret = msgBox.exec_()

    def handlerUploadResult(self, value):
        self.msgNotifySignal.emit("end:%s" % value)
        self.insertPlainText('![%s](%s)\n' % (value, value)),



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from PySide2.QtWidgets import QWidget,QStyle


class LineNumberArea(QWidget):

    def __init__(self, scriptEditor):
        super(LineNumberArea, self).__init__(scriptEditor)

        self.scriptEditor = scriptEditor
        self.setStyleSheet("text-align: center;")

    def paintEvent(self, event):
        self.scriptEditor.lineNumberAreaPaintEvent(event)
        return



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from PySide2.QtWebEngineWidgets import QWebEngineView, QWebEnginePage
from PySide2.QtCore import QUrl
import webbrowser
import PySide2


class WebEnginePage(QWebEnginePage):

    def javaScriptConsoleMessage(self, level, message, lineNumber,
                                 sourceID):  # real signature unknown; restored from __doc__
        """ javaScriptConsoleMessage(self, level: PySide2.QtWebEngineWidgets.QWebEnginePage.JavaScriptConsoleMessageLevel, message: str, lineNumber: int, sourceID: str) """
        print("javaScriptConsoleMessageCall#message = %s" % (message))

    def acceptNavigationRequest(self, url, _type, isMainFrame):
        if _type == QWebEnginePage.NavigationTypeLinkClicked:
            webbrowser.open(url.url(), new=1, autoraise=True)
            return False
        return True

    # 处理https错误提示
    def certificateError(self, certificateError: PySide2.QtWebEngineWidgets.QWebEngineCertificateError) -> bool:
        return True


class PreView(QWebEngineView):
    def __init__(self, *args, **kwargs):
        super(PreView, self).__init__(*args, **kwargs)
        # self.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Expanding)
        self.setPage(WebEnginePage(self))
        self.loadFinishStatus = False
        self.needCall = list()
        self.preContent = None
        self.loadFinished.connect(self.loadFinishCall)

        self.percentageOfTop = None

    def initHtml(self):
        self.setHtml('''<!DOCTYPE html>
<html lang="zh-cn">
<head>
    <meta charset="UTF-8"/>
    <meta content="width=device-width, initial-scale=1.0" name="viewport"/>
    <meta content="IE=edge, chrome=1" http-equiv="X-UA-Compatible"/>
    <meta content="text/html; charset=utf-8" http-equiv="content-type"/>
    <link href="https://cdn.bootcss.com/github-markdown-css/3.0.1/github-markdown.min.css" rel="stylesheet">
    <link rel="stylesheet"
          href="https://cdn.jsdelivr.net/gh/highlightjs/cdn-release@9.16.2/build/styles/default.min.css">
    <script type="text/javascript"
            src="https://cdn.jsdelivr.net/gh/highlightjs/cdn-release@9.16.2/build/highlight.min.js"></script>
    <script type="text/javascript" src="https://cdnjs.cloudflare.com/ajax/libs/mermaid/7.1.2/mermaid.min.js"></script>
    <script type="text/javascript"
            src="https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.5/MathJax.js?config=TeX-MML-AM_CHTML"></script>
    <script type="text/javascript">
        MathJax.Hub.Config({
            tex2jax: {
                inlineMath: [
                    ['$', '$'],
                    ['(', ')']
                ]
            }
        });
    </script>
    <style>
        img {
            max-width: 100%;
        }
    </style>
</head>
<body id="preArea">
<div id="preMkContent">
</div>
</body>
</html>''')

    def urlChanged(self, *args, **kwargs):  # real signature unknown
        print("urlChanged# args:%s and kwargs:%s" % (args, kwargs))
        QWebEngineView.urlChanged(self, *args, **kwargs)

    def loadStarted(self, *args, **kwargs):  # real signature unknown
        print("loadStarted# args:%s and kwargs:%s" % (args, kwargs))
        QWebEngineView.loadStarted(self, *args, **kwargs)

    def loadFinishCall(self, *args, **kwargs):  # real signature unknown
        print("loadFinishCall# args:%s and kwargs:%s" % (args, kwargs))
        self.loadFinishStatus = True
        for call in self.needCall:
            call()
        self.needCall.clear()

    def setPreContent(self, data):

        self.preContent = data
        if self.loadFinishStatus:
            self.realLoadPreContent()
        else:
            self.needCall.append(self.realLoadPreContent)

    def realLoadPreContent(self):
        if self.loadFinishStatus:
            self.preContent = str(self.preContent).replace("'", "\\\'")
            self.preContent = str(self.preContent).replace('"', "\\\"")

            javascriptStr = 'window.document.getElementById("preMkContent").innerHTML = \"%s\"' % str(
                self.preContent).replace(
                "\n",
                "")
            print(javascriptStr)
            self.page().runJavaScript(javascriptStr)

    def scrollContent(self, percentageOfTop=None):
        self.percentageOfTop = percentageOfTop

        if self.loadFinishStatus:
            self.realScrollContent()
        else:
            self.needCall.append(self.realScrollContent)

    def realScrollContent(self):
        if self.loadFinishStatus:
            javascriptStr = '''var scrollingElement = (document.scrollingElement || document.body);
scrollingElement.scrollTop = scrollingElement.scrollHeight * %s;''' % self.percentageOfTop
        self.page().runJavaScript(javascriptStr)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from PySide2.QtWidgets import QWidget, QVBoxLayout, QTabWidget, QHBoxLayout, QToolBar, QMessageBox, QTabBar, \
    QStylePainter, QStyleOptionTab, QStyle
from PySide2.QtCore import QMargins, QFileInfo, Qt
from PySide2.QtGui import QTextDocument
from .DataClass import FileInfo
from .MkEdit import MkEdit
from menusbar.core import MemusBar
import os


class EditCenter(QWidget):
    def __init__(self, parent=None):
        print("EditCenter __init__")

        QWidget.__init__(self, parent)
        self.init()

    def startInit(self):
        self._initUI()
        self._initData()
        self._initEvent()

    def __del__(self):
        print("EditCenter __del__")

    def init(self):
        self.toolbar: QToolBar = None

    def _initEvent(self):
        self.getMenusBar.openFileSignal.connect(self.openFileCallBack)
        self.getMenusBar.handlerBeforeEvent = self.handlerBeforeEvent
        self.getMenusBar.handlerAfterEvent = self.handlerAfterEvent

    def handlerBeforeEvent(self, type: int, filePath: str) -> bool:

        if type == self.getMenusBar.HANLDER_DELETE_FILE:
            msgBox = QMessageBox()
            msgBox.setFixedWidth(200)
            msgBox.setFixedHeight(200)
            msgBox.setText("删除提示")
            msgBox.setInformativeText("删除后将无法恢复，确认要删除：%s 文件？" % os.path.basename(filePath))
            msgBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
            msgBox.setDefaultButton(QMessageBox.Yes)
            ret = msgBox.exec_()
            if ret == QMessageBox.Yes:
                for index in range(0, self.mdiare.count()):
                    tab = self.mdiare.widget(index)
                    if tab.property("title") == filePath:
                        self.mdiare.removeTab(index)
            elif ret == QMessageBox.No:
                return False



        elif type == self.getMenusBar.HANLDER_DELETE_DIR:
            msgBox = QMessageBox()
            msgBox.setFixedWidth(200)
            msgBox.setFixedHeight(200)
            msgBox.setText("删除提示")
            msgBox.setInformativeText("将删除该目录下所有文件，确认要删除：%s 目录？" % os.path.basename(filePath))
            msgBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
            msgBox.setDefaultButton(QMessageBox.Yes)
            ret = msgBox.exec_()
            if ret == QMessageBox.Yes:
                for index in range(0, self.mdiare.count()):
                    tab = self.mdiare.widget(index)
                    if str(tab.property("title")).startswith(filePath):
                        self.mdiare.removeTab(index)
            elif ret == QMessageBox.No:
                return False

        return True

    def handlerAfterEvent(self, type: int, filePath: str):
        pass

    def openFileCallBack(self, file: QFileInfo):
        if file.isFile():
            fileInfo = FileInfo(file.fileName(), file.filePath())
            self.openFileInfo(fileInfo)

    def openFileInfo(self, value: FileInfo):
        has = False
        for index in range(0, self.mdiare.count()):
            tab = self.mdiare.widget(index)
            if tab and tab.property("title") == value.filePath:
                self.mdiare.setCurrentIndex(index)
                has = True
                break
        if not has:
            edit = MkEdit(self)
            edit.loadData(value.filePath)
            edit.setProperty("title", value.filePath)
            self.mdiare.addTab(edit, value.fileName)
            self.mdiare.setTabToolTip(self.mdiare.count() - 1, value.filePath)
            self.mdiare.setCurrentIndex(self.mdiare.count() - 1)

    def getCurrentDocment(self) -> QTextDocument:
        edit: MkEdit = self.mdiare.currentWidget()
        return edit.getDocument()

    def getCurrentEdit(self) -> MkEdit:
        edit: MkEdit = self.mdiare.currentWidget()
        return edit

    def tabCloseRequested(self, index):
        _currentTab: MkEdit = self.mdiare.widget(index)
        if not _currentTab.interceptClose():
            self.mdiare.removeTab(index)

    def tabBarClicked(self, index):
        print("tabBarClicked index : %s" % index)

    def tabBarDoubleClicked(self, index):
        print("tabBarDoubleClicked index : %s" % index)

    def currentTabChanged(self, index):
        print("currentTabChanged index : %s" % index)

    def currentChanged(self, index):
        print("currentChanged index : %s" % index)

    def destroy(self, destroyWindow: bool = ..., destroySubWindows: bool = ...):
        print("destroy")
        QWidget(EditCenter, self).destroy(destroyWindow, destroySubWindows)

    def create(self, arg__1: int = ..., initializeWindow: bool = ..., destroyOldWindow: bool = ...):
        print("create")
        QWidget(EditCenter, self).create(arg__1, initializeWindow, destroyOldWindow)

    def _initUI(self):
        self.mainLayout = QHBoxLayout(self)
        self.mainLayout.setContentsMargins(QMargins(0, 0, 0, 0))
        self.menuBar = MemusBar(self)
        self.mainLayout.addWidget(self.menuBar)
        self.rightLayout = QVBoxLayout(self)
        self.rightLayout.setContentsMargins(QMargins(0, 0, 0, 0))

        self.mainTab = QTabWidget(self)
        self.mainTab.setMovable(True)
        self.mainTab.currentChanged.connect(self.currentTabChanged)
        self.mainTab.setTabPosition(QTabWidget.West)
        self.mdiare = QTabWidget(self)
        self.mdiare.setDocumentMode(True)
        # 注册关闭按钮信号
        self.mdiare.tabCloseRequested.connect(self.tabCloseRequested)
        self.mdiare.tabBarClicked.connect(self.tabBarClicked)
        self.mdiare.tabBarDoubleClicked.connect(self.tabBarDoubleClicked)
        self.mdiare.currentChanged.connect(self.currentChanged)
        self.mdiare.setMovable(True)
        self.mdiare.setTabsClosable(True)

        if len(self.extendTabWidget()) <= 0:
            self.rightLayout.addWidget(self.mdiare)
        else:
            self.mainTab.addTab(self.mdiare, "编辑中心")
            for node in self.extendTabWidget():
                name, widget = node
                self.mainTab.addTab(widget, name)

            self.rightLayout.addWidget(self.mainTab)
            self.mainTab.setCurrentIndex(0)

        self.mainLayout.addLayout(self.rightLayout, 1)
        self.setLayout(self.mainLayout)

    def _initData(self):
        pass
        # edit = Edit(self)
        # self.mdiare.addTab(edit, "xxxx")
        # for i in range(10):

    def setToolBar(self, toolBar: QToolBar):
        if toolBar:
            if self.toolbar:
                self.rightLayout.removeWidget(self.toolbar)
            self.toolbar = toolBar
            self.rightLayout.insertWidget(0, self.toolbar)

    def extendTabWidget(self) -> list:
        return list()

    @property
    def getMenusBar(self):
        return self.menuBar



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PySide2.QtWidgets import QVBoxLayout, QWidget, QHBoxLayout, QMessageBox, QShortcut, QToolBar
from mistune import Markdown, Renderer
from PySide2.QtGui import QFont, QFontMetrics, QKeySequence, QTextDocument, QTextCursor, QTextOption
from PySide2.QtCore import QMargins
from os import path, stat
from rx import operators as ops, scheduler
from .FileUtils import updateFile
import PySide2
import rx
from scheduler.PySild2QtScheduler import qtScheduler
from .widgets import PreView, EditView, ToolBarButton, ToolBarMenuInfo
from .dialog import PrewViewDialog
from .MkUtuls import MkUtuls


class MkEdit(QWidget):
    def __init__(self, parent=None):
        print("EditWidget __init__")
        QWidget.__init__(self, parent)
        self.init()
        self.initUI()
        self.initEvent()
        self.initToolBar()
        self.refreshToolBar()

    def init(self):
        self.mkUtuls = MkUtuls()
        self.toolBar: QToolBar = QToolBar(self)
        self.toolBarMenusList = list()
        self.enableRightPreView: bool = False
        self.rightPreView = None
        self.leftEdit = None
        self.lastModifyTime = None
        self.timerCount = 0
        self.firstVisitLine = 1;
        self.scrollUp = False

    def initUI(self):
        self.mainLayout = QVBoxLayout(self)
        self.mainLayout.setContentsMargins(QMargins(0, 0, 0, 0))
        self.contentLayout = QHBoxLayout(self)
        self.contentLayout.setSpacing(0)
        self.initLeftEdit()
        if self.leftEdit:
            self.contentLayout.addWidget(self.leftEdit, 1)
        self.enableRightView(self.enableRightPreView)
        if self.toolBar:
            self.mainLayout.addWidget(self.toolBar)
        self.mainLayout.addLayout(self.contentLayout, 1)
        self.setLayout(self.mainLayout)

    def initLeftEdit(self):
        self.leftEdit = EditView(self)
        self.leftEdit.setWordWrapMode(QTextOption.WordWrap)
        shortcut = QShortcut(QKeySequence("Ctrl+S"), self.leftEdit)
        shortcut.activated.connect(self.save)
        font = QFont("Menlo", 14)
        self.leftEdit.setFont(font)
        self.leftEdit.setTabStopWidth(4 * QFontMetrics(font).width(" "))
        self.leftEdit.textChanged.connect(self.leftEditTextChange)

    def initEvent(self):
        self.startTimerTask()
        self.leftEdit.scrollContentEvent = self.fireScroll



    def fireScroll(self, firstVisitLine, up):
        self.scrollUp = (up == 1)

    def checkPreViewScroll(self):
        if self.rightPreView:
            line = self.leftEdit.firstVisibleBlock().blockNumber()
            if self.firstVisitLine != line:
                self.firstVisitLine = line
                if self.scrollUp:
                    # 由下往上滑
                    self.rightPreView.scrollContent(
                        (self.firstVisitLine + self.leftEdit.getShowLines()) / self.leftEdit.blockCount())
                else:
                    self.rightPreView.scrollContent(self.firstVisitLine / self.leftEdit.blockCount())

    # 执行定时器
    def startTimerTask(self):
        rx.interval(1).pipe(
            qtScheduler.QtScheduler()
        ).subscribe(
            on_next=lambda v: self.onTimer()
        )

    # 处理定时任务
    def onTimer(self):
        self.checkPreViewScroll()
        self.timerCount = self.timerCount + 1
        if self.timerCount % 10 == 0:
            self.trySave()

    def trySave(self):
        # print("trySave window is active:%s" % self.isActiveWindow())
        if not self.isActiveWindow():
            return
        if self.checkFileChange():
            # 提示用户源文件已发生改变
            msgBox = QMessageBox()
            msgBox.setText("%s-源文档已被修改" % self.fileName)
            msgBox.setInformativeText("点击YES将更新改动内容至当前文档,点击NO将以当前文本内容覆盖本地内容")
            msgBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
            msgBox.setDefaultButton(QMessageBox.Yes)
            ret = msgBox.exec_()
            if ret == QMessageBox.Yes:
                self.loadData(self.filePath)

            elif ret == QMessageBox.No:
                self.startSave()

    def initToolBar(self):
        self.toolBarMenusList.clear()
        self.toolBarMenusList.append(
            ToolBarMenuInfo("undo", "fa5s.undo-alt", shortcut="Ctrl+Z", callBack=lambda: self.leftEdit.undo()))

        self.toolBarMenusList.append(ToolBarMenuInfo("redo", "fa5s.redo-alt", callBack=lambda: self.leftEdit.redo()))

        self.toolBarMenusList.append(ToolBarMenuInfo("save", "fa5s.save", callBack=lambda: self.save()))

        self.toolBarMenusList.append(ToolBarMenuInfo(separator=True))

        self.toolBarMenusList.append(ToolBarMenuInfo("bold", "fa5s.bold"))

        self.toolBarMenusList.append(ToolBarMenuInfo("italic", "fa5s.italic"))

        self.toolBarMenusList.append(ToolBarMenuInfo("heading", "fa5s.heading"))
        self.toolBarMenusList.append(ToolBarMenuInfo(separator=True))

        self.toolBarMenusList.append(ToolBarMenuInfo("quote", "fa5s.quote-left"))

        self.toolBarMenusList.append(ToolBarMenuInfo("list-ul", "fa5s.list-ul"))

        self.toolBarMenusList.append(ToolBarMenuInfo("list-ol", "fa5s.list-ol"))

        self.toolBarMenusList.append(ToolBarMenuInfo(separator=True))

        self.toolBarMenusList.append(ToolBarMenuInfo("link", "fa5s.link"))
        self.toolBarMenusList.append(ToolBarMenuInfo("images", "fa5s.images"))
        self.toolBarMenusList.append(ToolBarMenuInfo(separator=True))

        self.toolBarMenusList.append(
            ToolBarMenuInfo("eye", "fa5s.eye", shortcut='Ctrl+W', callBack=lambda: self.showPreDialog()))
        self.toolBarMenusList.append(ToolBarMenuInfo("exchange", "fa5s.exchange-alt", shortcut='Ctrl+E',
                                                     callBack=lambda: self.enableRightView(
                                                         not self.enableRightPreView)))
        # self.toolBarMenusList.append(ToolBarMenuInfo("arrows", "fa5s.arrows-alt"))
        self.toolBarMenusList.append(ToolBarMenuInfo(separator=True))

        self.toolBarMenusList.append(ToolBarMenuInfo("question", "fa5s.question-circle"))

    def refreshToolBar(self):
        self.toolBar.clear()

        for data in self.toolBarMenusList:
            if data.separator:
                self.toolBar.addSeparator()
            else:
                menu = ToolBarButton()
                menu.setAwesomeIcon(data.icon)
                menu.setProperty("id", data.id)
                menu.clicked.connect(self.clickToolBarMenu)

                if data.shortcut:
                    shortcut = QShortcut(QKeySequence(data.shortcut), self)
                    shortcut.setProperty("id", data.id)
                    shortcut.setProperty("shortcut", data.shortcut)
                    shortcut.activated.connect(self.fireToolBarMenusShortcut)
                self.toolBar.addWidget(menu)

    def addExtendToolBar(self, index, button: ToolBarMenuInfo):
        self.toolBarMenusList.insert(index, button)

    def fireToolBarMenusShortcut(self):
        shortcut = self.sender().property("shortcut")
        print("fireToolBarMenusShortcut# shortcut=%s" % shortcut)
        for menu in self.toolBarMenusList:
            if menu.shortcut and menu.shortcut == shortcut:
                if menu.callBack:
                    menu.callBack()

    def clickToolBarMenu(self):
        id = self.sender().property("id")
        print("clickToolBarMenu# id=%s" % id)

        for menu in self.toolBarMenusList:
            if menu.id and menu.id == id:
                if menu.callBack:
                    menu.callBack()

    def showPreDialog(self):
        dialog = PrewViewDialog(self)
        dialog.setPreHtml(self.getRenderHtml(self.leftEdit.toPlainText()))

    def enableRightView(self, enable):
        if self.enableRightPreView != enable:
            self.enableRightPreView = enable
            if self.enableRightPreView:
                if not self.rightPreView:
                    self.rightPreView = PreView(self)
                    self.rightPreView.initHtml()
                    self.contentLayout.addWidget(self.rightPreView, 1)
                    self.foreRefreshPreViewInfo()

            else:
                if self.rightPreView:
                    self.contentLayout.removeWidget(self.rightPreView)
                    self.rightPreView.deleteLater()
                    del self.rightPreView
                    self.rightPreView = None

    def interceptClose(self):
        if path.exists(self.filePath) and not self.haseSave:
            msgBox = QMessageBox()
            msgBox.setText("%s-文档已被修改" % self.fileName)
            msgBox.setInformativeText("是否保存修改后的文档")
            msgBox.setStandardButtons(QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel)
            msgBox.setDefaultButton(QMessageBox.Save)
            ret = msgBox.exec_()
            if ret == QMessageBox.Save:
                self.save()
                self.haseSave = True
            elif ret == QMessageBox.Cancel:
                self.haseSave = False
            elif ret == QMessageBox.Discard:
                return False

        return not self.haseSave

    def save(self):
        if not self.haseSave:
            if not path.exists(self.filePath):
                msgBox = QMessageBox()
                msgBox.setText("%s-文档不存在" % self.fileName)
                msgBox.setInformativeText("请检查是否文件已被删除或转移至其他目录")
                msgBox.setStandardButtons(QMessageBox.Close)
                msgBox.setDefaultButton(QMessageBox.Close)
            else:
                self.startSave()

    def startSave(self):
        updateFile(self.filePath, self.leftEdit.toPlainText()).pipe(
            ops.subscribe_on(scheduler.ThreadPoolScheduler())
        ).subscribe(on_completed=lambda: self.handlerSaveResult())

    def handlerSaveResult(self):
        self.haseSave = True
        self.setFileLastModifyTime()

    def loadData(self, filePath):
        self.filePath = filePath
        self.fileName = path.basename(self.filePath)
        with open(self.filePath, mode='r') as f:
            self.leftEdit.setPlainText("".join(f.readlines()))
        self.setFileLastModifyTime()
        self.haseSave = True

    def setFileLastModifyTime(self):
        if path.exists(self.filePath):
            self.lastModifyTime = stat(self.filePath).st_mtime

    def checkFileChange(self) -> bool:
        try:
            currentModifyTime = stat(self.filePath).st_mtime
            if currentModifyTime != self.lastModifyTime:
                return True
            else:
                return False
        except:
            return False

    def setPlainText(self, text: str = ""):
        self.leftEdit.setPlainText(text)

    def getDocument(self) -> QTextDocument:
        return self.leftEdit.document()

    def leftEditTextChange(self):
        self.haseSave = False
        if self.rightPreView:
            self.rightPreView.setPreContent(self.getRenderHtml(self.leftEdit.toPlainText()))
            line = self.leftEdit.textCursor().blockNumber()

            if line > self.leftEdit.blockCount() - self.leftEdit.getShowLines():
                self.rightPreView.scrollContent(1.0)

    def foreRefreshPreViewInfo(self):
        if self.rightPreView:
            self.rightPreView.setPreContent(self.getRenderHtml(self.leftEdit.toPlainText()))

    def getRenderHtml(self, txt):
        result = self.mkUtuls.parse(txt)
        # print(result)
        return result

    def enterEvent(self, event: PySide2.QtCore.QEvent):
        self.trySave()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from abc import ABCMeta, abstractmethod, abstractproperty
import typing
import os
import logging
import time
import rx
import requests
import shutil
from qiniu import Auth, put_file
from PIL import Image, ImageDraw
import sys
import json
from PySide2.QtCore import QMimeData
from PySide2.QtGui import QImage

import tinify


def fileNameGenerate(originName) -> str:
    suffix = os.path.splitext(originName)[-1]
    today = time.strftime("%Y%m%d%H%M%S", time.localtime())
    newFileName = "%s_%s" % (today, originName)
    print("new file name: %s" % newFileName)
    return newFileName


def getProjectPath():
    return os.path.dirname(sys.argv[0])


class Strategy(metaclass=ABCMeta):

    def __init__(self):
        self.supportSuffixs = None
        self.saveDirPath = "cacheTemp"
        self.defaultSupportSuffixs: typing.List = ['.jpg', '.png', '.jpeg', '.gif', '.svg', '.bmp', '.webp']
        self.textWatermark = None
        self.tinifyAppKey = None

    def save(self, imageSrc: str = None) -> typing.Any:
        if not os.path.isfile(imageSrc):
            printError("imageSrc must is image file")
            return None
        baseFileName = os.path.basename(imageSrc)
        suffix = os.path.splitext(baseFileName)[-1]
        if str(suffix).lower() not in self.supportSuffixs:
            printError("current setting is not support for image's type of suffix :%s " % suffix)
            return None
        newFileName = fileNameGenerate(baseFileName)
        savePath = os.path.join(self.saveDirPath, newFileName)
        shutil.copyfile(imageSrc, savePath)
        # 1、压缩处理
        if self.tinifyAppKey:
            savePath = self.processCompression(savePath)

        # 2、水印处理
        if self.textWatermark:
            savePath = self.processWatermark(savePath)
        # 3、开始上传
        result, error = self.startSave(savePath)
        os.remove(savePath)
        if result:
            return {'code': 'success', 'data': result}
        else:
            return {'code': 'error', 'msg': error}

    def processCompression(self, imgStr: str) -> str:
        tinify.key = self.tinifyAppKey
        source = tinify.from_file(imgStr)
        source.to_file(imgStr)
        return imgStr

    def processWatermark(self, imgStr: str) -> str:
        return self.addTextToImage(imgStr, self.textWatermark)

    def addTextToImage(self, imgSrc, text) -> str:
        # font = ImageFont.load("simsun.pil")
        # font = ImageFont.truetype("simsun.ttc", 40, encoding="unic")  # 设置字体
        type = ""
        type_value = None
        if str(os.path.splitext(imgSrc)[-1]).lower() in [".jpeg", '.jpg']:
            image = Image.open(imgSrc)
            print(os.path.splitext(imgSrc)[0])
            os.remove(imgSrc)
            imgSrc = imgSrc.replace(os.path.splitext(imgSrc)[-1], ".png")
            image.save(imgSrc)

        image = Image.open(imgSrc)

        type = "RGBA"
        type_value = (255, 255, 255, 0)

        rgba_image = image.convert(type)
        text_overlay = Image.new(type, rgba_image.size, type_value)
        image_draw = ImageDraw.Draw(text_overlay)

        text_size_x, text_size_y = image_draw.textsize(text)
        # 设置文本文字位置
        print(rgba_image)
        text_xy = (rgba_image.size[0] - text_size_x - 8, rgba_image.size[1] - text_size_y - 10)
        # 设置文本颜色和透明度
        image_draw.text(text_xy, text, fill=(76, 234, 124))

        image_with_text = Image.alpha_composite(rgba_image, text_overlay)
        image_with_text.save(imgSrc)
        return imgSrc

    @abstractmethod
    def startSave(self, imageSrc: str = None) -> typing.Any:
        pass

    @abstractmethod
    def initSetting(self, param: dict = None) -> typing.Any:
        if 'tempCache' in param.keys():
            self.saveDirPath = param["tempCache"]
        else:
            self.saveDirPath = os.path.join(getProjectPath(), self.saveDirPath)
        if not os.path.exists(self.saveDirPath):
            os.mkdir(self.saveDirPath)

        if 'supportSuffixs' in param.keys():
            self.supportSuffixs = param["supportSuffixs"]
        else:
            self.supportSuffixs = self.defaultSupportSuffixs

        if 'textWatermark' in param.keys():
            self.textWatermark = param['textWatermark']

        if 'tinifyAppKey' in param.keys():
            self.tinifyAppKey = param['tinifyAppKey']
        self.init = True


def printError(msg: str = ""):
    logging.error("%s | %s " % ("ImageLocalSaveStrategy", msg))


class ImageQiniuSaveStrategy(Strategy):

    def __init__(self):
        Strategy.__init__(self)
        self.access_key = ""
        self.secret_key = ""
        self.bucket_name = ""
        self.host_image = ""

    def startSave(self, imageSrc: str = None) -> typing.Any:
        q = Auth(self.access_key, self.secret_key)
        token = q.upload_token(self.bucket_name, os.path.basename(imageSrc), 3600)

        ret, info = put_file(token, os.path.basename(imageSrc), imageSrc)
        print(info)
        print(ret)
        if info.status_code == 200:
            return ("%s/%s" % (self.host_image, ret['key']), None)
        return (None, "上传失败")

    def initSetting(self, param: dict = None) -> typing.Any:
        Strategy.initSetting(self, param)
        if 'access_key' in param.keys():
            self.access_key = param["access_key"]
        if 'secret_key' in param.keys():
            self.secret_key = param["secret_key"]
        if 'bucket_name' in param.keys():
            self.bucket_name = param["bucket_name"]

        if 'host_image' in param.keys():
            self.host_image = param["host_image"]


class ImageSMMSSaveStrategy(Strategy):

    def __init__(self):
        Strategy.__init__(self)
        self.appk = ""
        self.username = ""
        self.password = ""
        self.getTokenUrl = 'https://sm.ms/api/v2/token'
        self.uploadUrl = 'https://sm.ms/api/v2/upload'

    def startSave(self, imageSrc: str = None) -> typing.Any:
        return self.upload(imageSrc)

    def getToken(self):
        payload = {'username': self.username, 'password': self.password}
        response = requests.post(url=self.getTokenUrl, data=payload)
        rsp = response.text
        result = json.loads(rsp)
        if result['code'] == 'success':
            return result['data']['token']
        return None

    def upload(self, filePath):
        token = self.getToken()
        headers = {}
        if token:
            headers = {"Authorization": "Basic %s" % token}
            print(headers)
        files = {'smfile': open(filePath, 'rb')}

        response = requests.post(url=self.uploadUrl, headers=headers, files=files)
        rsp = response.text
        print(rsp)
        result = json.loads(rsp)
        if result['code'] == 'success':
            return (result['data']['url'], None)
        elif result['code'] == 'image_repeated':
            return (result['images'], None)
        else:
            return (None, result['message'])

    def initSetting(self, param: dict = None) -> typing.Any:
        Strategy.initSetting(self, param)
        if 'appk' in param.keys():
            self.appk = param["appk"]

        if 'username' in param.keys():
            self.username = param["username"]

        if 'password' in param.keys():
            self.password = param["password"]


class ImageUploadHelper:
    _instance = None

    def __init__(self):
        self.strategy: Strategy = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(ImageUploadHelper, cls).__new__(cls, *args, **kwargs)

        return cls._instance

    def init(self, strategy: Strategy = None):
        self.strategy = strategy

    def saveImage(self, imageSrcList: list = None) -> rx.Observable:

        _imageSrcList = imageSrcList

        def _createObserver(observable, scheduler):
            try:
                for img in _imageSrcList:
                    if self.strategy:
                        result = self.strategy.save(img)
                        if result["code"] == 'success':
                            observable.on_next(result['data'])
                        else:
                            observable.on_error(result["msg"])
                    else:
                        observable.on_error("未设置图片上传插件")
            except Exception as e:
                observable.on_error(e)

        return rx.create(_createObserver)

    def saveImageForQMineImage(self, data: QMimeData) -> rx.Observable:
        _data = data

        def _createObserver(observable, scheduler):
            tempFileName = None
            try:
                image = QImage(data.imageData())
                tempFileName = os.path.join(getProjectPath(), fileNameGenerate('%s.png' % time.time()))
                image.save(tempFileName)
                result = None
                if self.strategy:
                    result = self.strategy.save(tempFileName)
                    if result["code"] == 'success':
                        observable.on_next(result['data'])
                    else:
                        observable.on_error(result["msg"])
                else:
                    observable.on_error("未设置图片上传插件")
            except Exception as e:
                observable.on_error(e)
            finally:
                if tempFileName:
                    os.remove(tempFileName)

        return rx.create(_createObserver)


imageUploadManager = ImageUploadHelper()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PySide2.QtWidgets import QPushButton
import qtawesome as qta


class ToolBarButton(QPushButton):
    def __init__(self, parent=None):
        QPushButton.__init__(self, parent)

    def setAwesomeIcon(self, iconName):
        icon = qta.icon(iconName,
                        active=iconName,
                        color='gray',
                        color_active='blue')
        self.setIcon(icon)


class ToolBarMenuInfo:
    def __init__(self, id: str = None, icon: str = None, shortcut: str = None, separator: bool = False, callBack=None):
        self.id = id
        self.icon = icon
        self.shortcut = shortcut
        self.separator = separator
        self.callBack = callBack



from PySide2.QtWidgets import QDialog, QLineEdit, QLabel, QPushButton, QPlainTextEdit
from PySide2.QtGui import QTextDocument, QTextCursor
from PySide2 import QtWidgets
from PySide2.QtCore import QSize, Signal, QRegularExpression


class SearchHitDialog(QDialog):
    _instance = None

    _object = None

    @classmethod
    def getInstance(cls):
        if not cls._object:
            cls._object = SearchHitDialog()
        return cls._object

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(SearchHitDialog, cls).__new__(cls, *args, **kwargs)

        return cls._instance

    def init(self, edit: QPlainTextEdit):
        self.edit = edit

    def __init__(self):
        QDialog.__init__(self)
        self.setWindowTitle("检索")
        self.initView()
        size = QSize()
        size.setHeight(200)
        size.setWidth(450)
        self.setFixedSize(size)

    def initView(self):
        label = QLabel()
        label.setText("搜索关键词：")
        self.input = QLineEdit()
        searchForString = QPushButton("字符搜索")
        searchForRegex = QPushButton("正则搜索")
        searchForRegex.clicked.connect(self.clickRegex)
        searchForString.clicked.connect(self.clickString)

        mainLayout = QtWidgets.QGridLayout()
        mainLayout.addWidget(label, 0, 0, 1, 1)
        mainLayout.addWidget(self.input, 0, 1, 1, 5)
        mainLayout.addWidget(searchForString, 2, 4, 1, 2)
        mainLayout.addWidget(searchForRegex, 3, 4, 1, 2)

        self.setLayout(mainLayout)

    def clickRegex(self):
        searchTxt = self.input.text()
        if len(searchTxt) <= 0:
            return
        expression = QRegularExpression(searchTxt)
        if not self.edit.find(expression):
            cursor: QTextCursor = self.edit.textCursor()
            cursor.movePosition(QTextCursor.Start)
            self.edit.setTextCursor(cursor)

    def clickString(self):
        searchTxt = self.input.text()
        if len(searchTxt) <= 0:
            return
        if not self.edit.find(searchTxt):
            cursor: QTextCursor = self.edit.textCursor()
            cursor.movePosition(QTextCursor.Start)
            self.edit.setTextCursor(cursor)
